@@ -53,9 +53,9 @@ const size_t		ATTR_MAX_DIMS = 7;
 
 /* 3-D dataset with fixed dimensions */
 const int SPACE1_RANK = 3;
-const int SPACE1_DIM1 = 3;
-const int SPACE1_DIM2 = 15;
-const int SPACE1_DIM3 = 13;
+const hsize_t SPACE1_DIM1 = 3;
+const hsize_t SPACE1_DIM2 = 15;
+const hsize_t SPACE1_DIM3 = 13;
 
 /* Object names */
 const H5std_string DSET1_NAME("Dataset1");
@@ -65,7 +65,7 @@ const H5std_string TYPE1_NAME("/Type");
 /* Attribute Rank & Dimensions */
 const H5std_string ATTR1_NAME("Attr1");
 const int ATTR1_RANK = 1;
-const int ATTR1_DIM1 = 3;
+const hsize_t ATTR1_DIM1 = 3;
 int attr_data1[ATTR1_DIM1]={512,-234,98123}; /* Test data for 1st attribute */
 
 // File attribute, using the same rank and dimensions as ATTR1_NAME's
@@ -74,8 +74,8 @@ const H5std_string FATTR2_NAME("File Attr2");
 
 const H5std_string ATTR2_NAME("Attr2");
 const int ATTR2_RANK = 2;
-const int ATTR2_DIM1 = 2;
-const int ATTR2_DIM2 = 2;
+const hsize_t ATTR2_DIM1 = 2;
+const hsize_t ATTR2_DIM2 = 2;
 int attr_data2[ATTR2_DIM1][ATTR2_DIM2]={{7614,-416},{197814,-3}}; /* Test data for 2nd attribute */
 
 const H5std_string ATTR3_NAME("Attr3");
@@ -123,7 +123,7 @@ static void test_attr_basic_write()
     hsize_t dims2[] = {ATTR1_DIM1};
     hsize_t dims3[] = {ATTR2_DIM1,ATTR2_DIM2};
     int     read_data1[ATTR1_DIM1]={0}; // Buffer for reading 1st attribute
-    int     i;
+    hsize_t i;
 
     // Output message about test being performed
     SUBTEST("Basic Attribute Writing Functions");
@@ -384,7 +384,7 @@ static void test_attr_getname()
 static void test_attr_rename()
 {
     int read_data1[ATTR1_DIM1]={0}; // Buffer for reading the attribute
-    int i;
+    hsize_t i;
 
     // Output message about test being performed
     SUBTEST("Checking for Existence and Renaming Attribute");
@@ -493,7 +493,7 @@ static void test_attr_rename()
 ********************************************************************/
 static void test_attr_basic_read()
 {
-    int i, j;
+    hsize_t i, j;
 
     // Output message about test being performed
     SUBTEST("Basic Attribute Reading Functions");
@@ -624,7 +624,6 @@ static void test_attr_compound_read()
     size_t      size;   // Attribute datatype size as stored in file
     size_t      offset; // Attribute datatype field offset
     struct attr4_struct read_data4[ATTR4_DIM1][ATTR4_DIM2]; // Buffer for reading 4th attribute
-    hsize_t i,j;
 
     // Output message about test being performed
     SUBTEST("Basic Attribute Functions");
@@ -654,7 +653,7 @@ static void test_attr_compound_read()
 
 	// Get the dims of the dataspace and verify them
 	int ndims = space.getSimpleExtentDims(dims);
-	if(dims[0]!=ATTR4_DIM1)
+	verify_val(ndims, ATTR4_RANK, "DataSpace::getSimpleExtentDims", __LINE__, __FILE__);
         verify_val((long)dims[0], (long)ATTR4_DIM1, "DataSpace::getSimpleExtentDims",__LINE__, __FILE__);
         verify_val((long)dims[1], (long)ATTR4_DIM2, "DataSpace::getSimpleExtentDims",__LINE__, __FILE__);
 
@@ -673,13 +672,14 @@ static void test_attr_compound_read()
 
 	// Verify that the fields have the same names as when the type
 	// was created
-	for(i=0; i<fields; i++)
+	int j;
+	for(j=0; j<fields; j++)
 	{
-	    H5std_string fieldname = datatype.getMemberName(i);
+	    H5std_string fieldname = datatype.getMemberName(j);
 	    if(!((fieldname == ATTR4_FIELDNAME1) ||
 		(fieldname == ATTR4_FIELDNAME2) ||
 		(fieldname == ATTR4_FIELDNAME3)))
-            TestErrPrintf("%d:invalid field name for field #%d: %s\n",__LINE__,i,fieldname.c_str());
+            TestErrPrintf("%d:invalid field name for field #%d: %s\n",__LINE__,j,fieldname.c_str());
 	} /* end for */
 
 	offset = datatype.getMemberOffset(0);
@@ -729,23 +729,38 @@ static void test_attr_compound_read()
 	attr.read(datatype, read_data4);
 
 	// Verify values read in
-	for(i=0; i<ATTR4_DIM1; i++)
-	    for(j=0; j<ATTR4_DIM2; j++)
-		if(HDmemcmp(&attr_data4[i][j],&read_data4[i][j],sizeof(struct attr4_struct))) {
-		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].i=%d, read_data4[%d][%d].i=%d\n",__LINE__,i,j,attr_data4[i][j].i,i,j,read_data4[i][j].i);
-		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].d=%f, read_data4[%d][%d].d=%f\n",__LINE__,i,j,attr_data4[i][j].d,i,j,read_data4[i][j].d);
-		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].c=%c, read_data4[%d][%d].c=%c\n",__LINE__,i,j,attr_data4[i][j].c,i,j,read_data4[i][j].c);
+	hsize_t ii, jj;
+	for(ii=0; ii<ATTR4_DIM1; ii++)
+	    for(jj=0; jj<ATTR4_DIM2; jj++)
+		if(HDmemcmp(&attr_data4[ii][jj],&read_data4[ii][jj],sizeof(struct attr4_struct))) {
+		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].i=%d, read_data4[%d][%d].i=%d\n",__LINE__,ii,jj,attr_data4[ii][jj].i,ii,jj,read_data4[ii][jj].i);
+		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].d=%f, read_data4[%d][%d].d=%f\n",__LINE__,ii,jj,attr_data4[ii][jj].d,ii,jj,read_data4[ii][jj].d);
+		    TestErrPrintf("%d:attribute data different: attr_data4[%d][%d].c=%c, read_data4[%d][%d].c=%c\n",__LINE__,ii,jj,attr_data4[ii][jj].c,ii,jj,read_data4[ii][jj].c);
              } /* end if */
 
 	// Verify name
 	H5std_string attr_name = attr.getName();
 	verify_val(attr_name, ATTR4_NAME, "Attribute::getName", __LINE__, __FILE__);
-	PASSED();
     } // end try block
 
     catch (Exception E) {
 	issue_fail_msg("test_attr_compound_read()", __LINE__, __FILE__, E.getCDetailMsg());
     }
+
+    try
+    {
+	// Now, try truncating the file to make sure reference counting is good.
+	// If any references to ids in the previous block are left unterminated,
+	// the truncating will fail, because the file will not be closed in
+	// the file.close() above.
+	H5File file1(FILE_COMPOUND, H5F_ACC_TRUNC);
+
+	PASSED();
+    } // end try block
+
+    catch (FileIException E) {
+	issue_fail_msg("test_attr_compound_read()", __LINE__, __FILE__, "Unable to truncate file, possibly because some objects are left opened");
+    }
 }   // test_attr_compound_read()
 
 /****************************************************************
@@ -928,7 +943,7 @@ static void test_attr_mult_read()
     int     read_data1[ATTR1_DIM1]={0}; // Buffer for reading 1st attribute
     int     read_data2[ATTR2_DIM1][ATTR2_DIM2]={{0}}; // Buffer for reading 2nd attribute
     double  read_data3[ATTR3_DIM1][ATTR3_DIM2][ATTR3_DIM3]={{{0}}}; // Buffer for reading 3rd attribute
-    int     i,j,k;
+    hsize_t i,j,k;
 
 	// Output message about test being performed
     SUBTEST("Multiple Attribute Reading Functions");
@@ -959,7 +974,7 @@ static void test_attr_mult_read()
 	// Get the dims of the dataspace and verify them
 	hsize_t dims[ATTR_MAX_DIMS];    // Attribute dimensions
 	int ndims = space.getSimpleExtentDims(dims);
-	if(dims[0]!=ATTR1_DIM1)
+	if ((long)dims[0] != (long)ATTR1_DIM1)
 	    TestErrPrintf("%d:attribute dimensions different: dims[0]=%d, should be %d\n",__LINE__,(int)dims[0],ATTR1_DIM1);
 
 	/* Verify Datatype */
@@ -1010,10 +1025,9 @@ static void test_attr_mult_read()
 
 	// Get the dims of the dataspace and verify them
 	ndims = space.getSimpleExtentDims(dims);
-	if(dims[0]!=ATTR2_DIM1)
-	    TestErrPrintf("%d:attribute dimensions different: dims[0]=%d, should be %d\n",__LINE__,(int)dims[0],ATTR2_DIM1);
-	if(dims[1]!=ATTR2_DIM2)
-	    TestErrPrintf("%d:attribute dimensions different: dims[1]=%d, should be %d\n",__LINE__,(int)dims[1],ATTR2_DIM2);
+
+        verify_val((long)dims[0], (long)ATTR2_DIM1, "DataSpace::getSimpleExtentDims",__LINE__, __FILE__);
+        verify_val((long)dims[1], (long)ATTR2_DIM2, "DataSpace::getSimpleExtentDims",__LINE__, __FILE__);
 
 	/* Verify Datatype */
 
@@ -1554,6 +1568,198 @@ static void test_attr_exists()
     }
 }   // test_attr_exists()
 
+/****************************************************************
+**
+**  test_attr_dense_create(): Test phase change properties
+**      Tests "dense" attribute storage creation
+**
+****************************************************************/
+const H5std_string FILE_CRTPROPS("tattr_crt_properties.h5");
+const int NAME_BUF_SIZE = 1024;
+const unsigned MAX_COMPACT_DEF = 8;
+const unsigned MIN_DENSE_DEF = 6;
+
+static void test_attr_dense_create(FileCreatPropList& fcpl,
+				   FileAccPropList& fapl)
+{
+    // Output message about test being performed
+    SUBTEST("Dense Attribute Storage Creation");
+
+    try {
+	// Create file
+	H5File fid1 (FILE_CRTPROPS, H5F_ACC_TRUNC, fcpl, fapl);
+
+	// Close file
+	fid1.close();
+
+	// Get size of file
+	h5_stat_size_t empty_filesize;       // Size of empty file
+	empty_filesize = h5_get_file_size(FILE_CRTPROPS.c_str(), fapl.getId());
+	if (empty_filesize < 0)
+            TestErrPrintf("Line %d: file size wrong!\n", __LINE__);
+
+	// Re-open file
+	fid1.openFile(FILE_CRTPROPS, H5F_ACC_RDWR, fapl);
+
+	// Create dataspace for dataset
+	DataSpace ds_space(H5S_SCALAR);
+
+	//  Create dataset creation property list.
+	DSetCreatPropList dcpl;
+
+	// Create a dataset
+	DataSet dataset = fid1.createDataSet(DSET1_NAME, PredType::NATIVE_UCHAR, ds_space, dcpl);
+
+	unsigned max_compact = 0, min_dense = 0;
+
+	// Retrieve limits for compact/dense attribute storage
+	dcpl.getAttrPhaseChange(max_compact, min_dense);
+	verify_val(max_compact, MAX_COMPACT_DEF, "DSetCreatPropList::getAttrPhaseChange",__LINE__,__FILE__);
+	verify_val(min_dense, MIN_DENSE_DEF, "DSetCreatPropList::getAttrPhaseChange",__LINE__,__FILE__);
+
+	// Set new compact/dense attribute storage limits to some random numbers
+	dcpl.setAttrPhaseChange(7, 5);
+
+	// Retrieve limits for compact/dense attribute storage and verify them
+	dcpl.getAttrPhaseChange(max_compact, min_dense);
+	verify_val(max_compact, 7, "DSetCreatPropList::getAttrPhaseChange",__LINE__,__FILE__);
+	verify_val(min_dense, 5, "DSetCreatPropList::getAttrPhaseChange",__LINE__,__FILE__);
+
+	// Close property list
+	dcpl.close();
+
+	// H5O_is_attr_dense_test - un-usable
+
+	// Add attributes, until just before converting to dense storage
+	char attr_name[NAME_BUF_SIZE];
+	unsigned attr_num;
+	for (attr_num = 0; attr_num < max_compact; attr_num++)
+	{
+	    // Create attribute
+	    sprintf(attr_name, "attr %02u", attr_num);
+	    Attribute attr = dataset.createAttribute(attr_name, PredType::NATIVE_UINT, ds_space);
+
+	    // Write data to the attribute
+	    attr.write(PredType::NATIVE_UINT, &attr_num);
+	} // end for
+
+	// H5O_is_attr_dense_test - un-usable
+
+	{ // Add one more attribute, to push into "dense" storage
+
+	    // Create another attribute
+	    sprintf(attr_name, "attr %02u", attr_num);
+	    Attribute attr = dataset.createAttribute(attr_name, PredType::NATIVE_UINT, ds_space);
+
+	    // Write data to the attribute
+	    attr.write(PredType::NATIVE_UINT, &attr_num);
+	}
+
+	// Attempt to add attribute again, which should fail
+	try
+	{
+	    // Create another attribute
+	    sprintf(attr_name, "attr %02u", attr_num);
+	    Attribute attr = dataset.createAttribute(attr_name, PredType::NATIVE_UINT, ds_space);
+
+	    // continuation here, that means no exception has been thrown
+	    throw InvalidActionException("DataSet::createAttribute", "Maximum number of attributes has been reached");
+	}
+	catch (AttributeIException E) // catching invalid action
+        {} // do nothing, exception expected
+
+	PASSED();
+    } // end try block
+
+    catch (Exception E) {
+	issue_fail_msg("test_attr_dense_create()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+}   // test_attr_dense_create()
+
+/****************************************************************
+**
+**  test_attr_corder_create_basic(): Test creation order properties
+**      Tests creating an object w/attribute creation order info
+**
+****************************************************************/
+static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
+				   FileAccPropList& fapl)
+{
+    // Output message about test being performed
+    SUBTEST("Basic Code for Attributes with Creation Order Info");
+
+    try {
+	// Create file
+	H5File fid1 (FILE_CRTPROPS, H5F_ACC_TRUNC, fcpl, fapl);
+
+	//  Create dataset creation property list.
+	DSetCreatPropList dcpl;
+
+	// Get creation order indexing on object
+	unsigned crt_order_flags = 0;
+	crt_order_flags = dcpl.getAttrCrtOrder();
+	verify_val(crt_order_flags, 0, "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+
+	// Setting invalid combination of a attribute order creation order
+	// indexing on should fail
+	try {
+	    dcpl.setAttrCrtOrder(H5P_CRT_ORDER_INDEXED);
+
+	    // continuation here, that means no exception has been thrown
+	    throw InvalidActionException("DSetCreatPropList::getAttrCrtOrder", "Indexing cannot be set alone, order tracking is required");
+	}
+	catch (PropListIException E) // catching invalid action
+        {} // do nothing, exception expected
+
+	// Set attribute creation order tracking & indexing for object then
+	// verify them
+	dcpl.setAttrCrtOrder(H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED);
+	crt_order_flags = dcpl.getAttrCrtOrder();
+	verify_val(crt_order_flags, (H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+
+	// Create dataspace for dataset
+	DataSpace ds_space(H5S_SCALAR);
+
+	// Create a dataset
+	DataSet dataset = fid1.createDataSet(DSET1_NAME, PredType::NATIVE_UCHAR, ds_space, dcpl);
+
+	// Close dataspace
+	ds_space.close();
+
+	// Check on dataset's attribute storage status.
+	// NOTE: Wrappers not available yet (H5O_is_attr_empty_test
+	// and H5O_is_attr_dense_test)
+
+	// Close dataset
+	dataset.close();
+
+	// Close property list
+	dcpl.close();
+
+	// Close file
+	fid1.close();
+
+	// Re-open file
+	fid1.openFile(FILE_CRTPROPS, H5F_ACC_RDWR, fapl);
+
+	// Open dataset created previously
+	dataset = fid1.openDataSet(DSET1_NAME);
+
+	// Retrieve dataset creation property list for the dataset
+	dcpl = dataset.getCreatePlist();
+
+	// Query the attribute creation properties
+	crt_order_flags = dcpl.getAttrCrtOrder();
+	verify_val(crt_order_flags, (H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+
+	PASSED();
+    } // end try block
+
+    catch (Exception E) {
+	issue_fail_msg("test_attr_corder_create_basic()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+}   // test_attr_corder_create_basic()
+
 /****************************************************************
 **
 **  test_attr(): Main attribute testing routine.
@@ -1568,26 +1774,80 @@ void test_attr()
     //MESSAGE("Testing Attributes\n");
     MESSAGE(5, ("Testing Attributes\n"));
 
-    test_attr_basic_write();	// Test basic H5A writing code
-    test_attr_getname();	// Test overloads of Attribute::getName
-    test_attr_rename();		// Test renaming attribute
-    test_attr_basic_read(); 	// Test basic H5A reading code
+    try
+    {
+        // Create a default file access property list
+        FileAccPropList fapl;
 
-    test_attr_compound_write();	// Test complex datatype H5A writing code
-    test_attr_compound_read();	// Test complex datatype H5A reading code
+        // Copy the file access property list for new format test
+        FileAccPropList fapl_new = fapl;
 
-    test_attr_scalar_write();	// Test scalar dataspace H5A writing code
-    test_attr_scalar_read();	// Test scalar dataspace H5A reading code
+        // Set the "use the latest version of the format" bounds for creating
+        // objects in the file
+        fapl_new.setLibverBounds(H5F_LIBVER_LATEST, H5F_LIBVER_LATEST);
 
-    test_attr_mult_write();	// Test writing multiple attributes
-    test_attr_mult_read();	// Test reading multiple attributes
-    test_attr_delete();		// Test deleting attributes
+        // Create a default file creation property list
+        FileCreatPropList fcpl;
 
-    test_attr_dtype_shared();	// Test using shared datatypes in attributes
+        // Copy the file creation property list for new format test
+        FileCreatPropList fcpl_new = fcpl;
 
-    test_string_attr();		// Test read/write string attribute
-    test_attr_exists();		// Test H5Location::attrExists
+        // Wrappers for ..._shared_mesg_nindexes are not available, skip
+        // use_shared test
 
+        // Loop over using new group format
+        hbool_t new_format;
+        for (new_format = FALSE; new_format <= TRUE; new_format++)
+        {
+            FileAccPropList curr_fapl;
+
+            // Set the file access proplist for the type of format
+            if (new_format)
+            {
+	    MESSAGE(7, ("testing with new file format\n"));
+	    curr_fapl = fapl_new;
+            }
+            else
+            {
+	    MESSAGE(7, ("testing with old file format\n"));
+	    curr_fapl = fapl;
+            }
+
+            test_attr_basic_write();	// Test basic H5A writing code
+            test_attr_getname();	// Test overloads of Attribute::getName
+            test_attr_rename();		// Test renaming attribute
+            test_attr_basic_read(); 	// Test basic H5A reading code
+
+            test_attr_compound_write();	// Test complex datatype H5A writing code
+            test_attr_compound_read();	// Test complex datatype H5A reading code
+
+            test_attr_scalar_write();	// Test scalar dataspace H5A writing code
+            test_attr_scalar_read();	// Test scalar dataspace H5A reading code
+
+            test_attr_mult_write();	// Test writing multiple attributes
+            test_attr_mult_read();	// Test reading multiple attributes
+            test_attr_delete();		// Test deleting attributes
+
+            test_attr_dtype_shared();	// Test using shared datatypes in attributes
+
+            test_string_attr();		// Test read/write string attribute
+            test_attr_exists();		// Test H5Location::attrExists
+
+            // Test with new format
+            if (new_format)
+            {
+		// Test dense attribute storage creation
+                test_attr_dense_create(fcpl, curr_fapl);
+
+		// Test create objects with attribute creation info
+                test_attr_corder_create_basic(fcpl, curr_fapl);
+            }
+        } // end for
+    } // end try block
+
+    catch (Exception E) {
+	issue_fail_msg("test_attr()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
 }   // test_attr()
 
 /*-------------------------------------------------------------------------
@@ -1614,5 +1874,6 @@ void cleanup_attr()
     HDremove(FILE_SCALAR.c_str());
     HDremove(FILE_MULTI.c_str());
     HDremove(FILE_DTYPE.c_str());
+    HDremove(FILE_CRTPROPS.c_str());
 }
 
